#include "ofMain.h"
#include "testApp.h"
#include "ofAppGlutWindow.h"

//========================================================================
int main( ){

    ofAppGlutWindow window;
	ofSetupOpenGL(&window, 2048, 768, OF_WINDOW);
    ofSetWindowTitle("Quase-Cinema 2");
    //ofSetWindowPosition(0,0);
	ofRunApp( new testApp());

}
